<?php

namespace App\Util;

use App\Exception\Exception;
use Doctrine\DBAL\Types\StringType;
use App\Test\ConnectionMock;

class Util
{
    private $connection;

    public function __construct(\Doctrine\DBAL\Connection $connection)
    {
        $this->connection = $connection;
        $this->logPath = $this->logPath();
        $this->logService = $this->logPath . '/Services';

    }

    private function logPath(){
        global $CLIENT_NAME;
        return $CLIENT_NAME['logPath'];
    }
    
    public function getCurrentTimezone($state)
    {
        $timezones = [
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Sao_Paulo",    "ES" => "America/Sao_Paulo",
            "GO" => "America/Sao_Paulo",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Sao_Paulo",    "PR" => "America/Sao_Paulo",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Sao_Paulo",    "RN" => "America/Fortaleza",
            "RS" => "America/Sao_Paulo",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Sao_Paulo",
            "SE" => "America/Maceio",       "SP" => "America/Sao_Paulo",
            "TO" => "America/Araguaia",
        ];

        return $timezones[$state];
    }

    public function get_uuid()
    {
        return sprintf(
            '%04x%04x-%04x-%04x-%04x-%04x%04x%04x',
            mt_rand(0, 0xffff),
            mt_rand(0, 0xffff),
            mt_rand(0, 0xffff),
            mt_rand(0, 0x0fff) | 0x4000,
            mt_rand(0, 0x3fff) | 0x8000,
            mt_rand(0, 0xffff),
            mt_rand(0, 0xffff),
            mt_rand(0, 0xffff)
        );
    }

    public function writeLog($type, $fileName, $content)
    {
        global $CLIENT_NAME;
        $config = $CLIENT_NAME;

        $basePath     = $this->logPath;

        switch ($type) {
            case 'order':
                $folder      = $basePath . "/orders/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder/$fileName" . ".txt";
                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));

                break;

            case 'order_pending':
                $folder      = $basePath . "/pending/";
                $currentFile = "$folder/$fileName";
                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }
                if ((file_exists($currentFile)))
                    return true;
                file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));

                break;

            case 'order_processed':
                $folder      = $basePath . "/processed/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder/$fileName";
                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));

                break;

            case 'order_error':
                $folder      = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder/$fileName"  . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }
                file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));

                break;

            case 'network':
                $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder/$fileName";
                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                file_put_contents($currentFile, $content);

                break;

            case 'execution':
                $folder      = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
                $currentFile = "$folder" . date("d") . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                file_put_contents($currentFile, $content . "\n", FILE_APPEND);

                break;

            case 'sendProducts_execution':
                $folder      = $basePath . "/Services/sendProducts/execution/" . date("Y") . "/" . date("m") . "/";
                $currentFile = "$folder" . date("d") . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                file_put_contents($currentFile, $content . "\n", FILE_APPEND);

                break;

            case 'sendProducts_error':
                $folder      = $basePath . "/Services/sendProducts/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder" . date("d") . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                file_put_contents($currentFile, $content, FILE_APPEND);

                break;

            case 'canceled_error':
                $folder      = $basePath . "/Services/canceled/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder" . date("d") . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                file_put_contents($currentFile, $content, FILE_APPEND);

                break;

            case 'canceled_pending':
                $folder      = $basePath . "/Services/canceled/pending/";
                $currentFile = "$folder" . $fileName . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }
                if ((file_exists($currentFile)))
                    return true;

                file_put_contents($currentFile, json_encode($content), FILE_APPEND);

                break;

            case 'canceled_processed':
                $folder      = $basePath . "/Services/canceled/processed/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder" . $fileName;

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                file_put_contents($currentFile, json_encode($content), FILE_APPEND);

                break;

            case 'canceled_received':
                $folder      = $basePath . "/Services/canceled/received/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder" . $fileName . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                if ((file_exists($currentFile)))
                    return true;

                file_put_contents($currentFile, json_encode($content), FILE_APPEND);

                break;

            case 'close_pending':
                $folder      = $basePath . "/Services/close/pending/";
                $currentFile = "$folder" . $fileName . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }
                if ((file_exists($currentFile)))
                    return true;

                file_put_contents($currentFile, json_encode($content), FILE_APPEND);

                break;

            case 'close_processed':
                $folder      = $basePath . "/Services/close/processed/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder" . $fileName;

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                file_put_contents($currentFile, json_encode($content), FILE_APPEND);

                break;

            case 'close_received':
                $folder      = $basePath . "/Services/close/received/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder" . $fileName . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                if ((file_exists($currentFile)))
                    return true;

                file_put_contents($currentFile, json_encode($content), FILE_APPEND);

                break;

            case 'transfer_pending':
                $folder      = $basePath . "/Services/transfer/pending/";
                $currentFile = "$folder" . $fileName . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }
                if ((file_exists($currentFile)))
                    return true;

                file_put_contents($currentFile, json_encode($content), FILE_APPEND);

                break;

            case 'transfer_processed':
                $folder      = $basePath . "/Services/transfer/processed/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder" . $fileName;

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                file_put_contents($currentFile, json_encode($content), FILE_APPEND);

                break;

            case 'transfer_received':
                $folder      = $basePath . "/Services/transfer/received/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
                $currentFile = "$folder" . $fileName . ".txt";

                if (!file_exists($folder)) {
                    mkdir($folder, 0775, true);
                }

                if ((file_exists($currentFile)))
                    return true;

                file_put_contents($currentFile, json_encode($content), FILE_APPEND);

                break;

            default:
                throw new Exception("Erro. Tipo de log {$type} não catalogado.");

                break;
        }

        return null;
    }

    public function formatNetworkError($networkData)
    {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../../templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response
        ];

        $data = str_replace(array_keys($templateParams), array_values($templateParams), $template);
        return str_replace("\n", "\r\n", $data);
    }

    public function request($method, $route, $body)
    {
        global $CLIENT_NAME;
        $config = $CLIENT_NAME;

        if (empty($config['abrahaoURL']))
            throw new Exception("URL Abrahao(oimenu) não informado. Favor verificar o arquivo de configuração.");

        if (empty($config['login_token']))
            throw new Exception("Token Abrahao(oimenu) não informado. Favor verificar o arquivo de configuração.");

        $abrahaoURL = $config['abrahaoURL'];
        $currentToken = $config['login_token'];
        $header = [
                'Content-Type: application/json',
                'Authorization: Bearer ' . $currentToken
            ];

        $curl = curl_init();
        curl_setopt_array($curl, [
            CURLOPT_URL             => $abrahaoURL . '/api/v1/' . $route,
            CURLOPT_RETURNTRANSFER  => true,
            CURLOPT_ENCODING        => '',
            CURLOPT_CONNECTTIMEOUT  => 5,
            CURLOPT_TIMEOUT         => 20,
            CURLOPT_FOLLOWLOCATION  => true,
            CURLOPT_SSL_VERIFYPEER  => false,
            CURLOPT_HTTP_VERSION    => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST   => $method,
            CURLOPT_POSTFIELDS      => !empty($body) ? json_encode($body) : null,
            CURLOPT_HTTPHEADER      => $header,
        ]);

        $curlResponse = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($curl)) || (($httpCode != 200) && ($httpCode != 201)) || (stripos($curlResponse, "<html>")) || (stripos($curlResponse, "Fatal error"))) {
            $fileName = "REQ-" . date("H-i-s") . ".txt";

            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $header,
                'url'       => $abrahaoURL . '/api/v1/' . $route,
                'method'    => $method,
                'params'    => json_encode($body),
                'httpCode'  => $httpCode,
                'error'     => curl_error($curl),
                'response'  => $curlResponse
            ]));
        }
        curl_close($curl);

        if (($httpCode != 200) && ($httpCode != 201))
            $this->writeLog("execution", "", "Erro (httpCode: $httpCode) \r\n");

        return $curlResponse;
    }

    public function formatNrSeqProdCom($seq)
    {
        return str_pad((string) ((int) $seq), 3, '0', STR_PAD_LEFT);
    }

    public function removeDecimal($value, $offset = 2)
    {
        return floatval(bcdiv($value, 1, $offset));
    }

    public function subtract($a, $b)
    {
        return floatval(bcsub($a, $b, 6));
    }

    public function add($a, $b)
    {
        return floatval(bcadd($a, $b, 6));
    }

    public function mult($a, $b)
    {
        return floatval(bcmul($a, $b, 6));
    }

    public function div($a, $b)
    {
        return floatval(bcdiv($a, $b, 6));
    }

    public function toRight($code, $size)
    {
        $code =  str_pad($code, $size, 0, STR_PAD_LEFT);
        return $code;
    }

    public function toFloat($number)
    {
        $number = str_replace(",", ".", $number);
        return floatval($number);
    }

    public function finish($file, $function, $removeFile = true)
    {
        $basePath     = $this->logPath;
        $date =
            $processedPath = "{$basePath}/$function/";

        if ($removeFile)
            unlink($processedPath . $file);
    }

    public function createFolder($folder)
    {
        if (!is_dir($folder)) {
            if (mkdir($folder, 0777, true)) {
                return true;
            } else {
                return false;
            }
        } else {
            return true;
        }
    }

    public function geraCodigo($contador, $qtde, $size, $mockNumber = false)
    {
        try {
            $sql = "
                    SELECT CDCONTADOR, NRSEQUENCIAL
                        FROM NOVOCODIGO
                            WHERE CDCONTADOR = :CDCONTADOR
                ";

            $params = array(
                'CDCONTADOR' => $contador
            );
            $sequencialAnterior = $this->connection->fetchAssoc($sql, $params);
            if (!method_exists($this->connection, 'getIsMocking') || (method_exists($this->connection, 'getIsMocking'))) {
                $referencia = "";
                $procedure = new StoredProcedure($this->connection, 'NOVO_CODIGO');
                $procedure->addParam(new Param('P_CONTADOR', Param::PARAM_INPUT, $contador, Param::PARAM_TYPE_STR));
                $procedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, $referencia, Param::PARAM_TYPE_STR, 20));
                $procedure->execute();
            }

            $sequencial = $this->connection->fetchAssoc($sql, $params);

            if (!empty($sequencialAnterior)) {
                if ($sequencialAnterior['NRSEQUENCIAL'] == $sequencial['NRSEQUENCIAL']) {
                    $sequencial['NRSEQUENCIAL'] = $sequencial['NRSEQUENCIAL'] + 1;

                    $sql = "
                        UPDATE NOVOCODIGO 
                            SET NRSEQUENCIAL = :NRSEQUENCIAL
                                WHERE CDCONTADOR = :CDCONTADOR
                        ";

                    $params = array(
                        'CDCONTADOR' => $contador,
                        'NRSEQUENCIAL' => str_pad($sequencial['NRSEQUENCIAL'], 20, '0', STR_PAD_LEFT),
                    );

                    $this->connection->executeQuery($sql, $params);
                }
            }

            $intSeq = (int)$sequencial['NRSEQUENCIAL'];
            return str_pad($intSeq, $size, "0", STR_PAD_LEFT);
        } catch (Exception $e) {
            throw new Exception('Erro ao executar procedure de novo código.', 1);
        }
    }

    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */

    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array())
    {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);

        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam(2, $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue(1, $param->getValue());
            }
        }
        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure)
    {
        $paramNames = array_map(function (Param $param) {
            //return " :" . $param->getName();
            return " ?"; // .$param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "EXEC " . $procedure->getName()  . $paramString;
    }
}
